﻿/*!
 *      @date   2011/09/08
 *      @file   dhwr_result.h
 *      @author shchoi
 *
 *      Copyright (c) 2016 by SELVAS AI Inc. All rights reserved.
 */

#ifndef __DHWR_RESULT_OBJECT_H__
#define __DHWR_RESULT_OBJECT_H__ 1

#include "dhwr_define.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @defgroup ResultObject ResultObject APIs
 * @ingroup  Main
 * @brief    인식된 결과를 저장하는 ResultObject 관련 함수
 * @details     DHWRCreateResultObject()로 생성된 오브젝트는 반드시 DHWRDestroyResultObject()를 호출하여 소멸시켜야 한다.
 * - 블럭 오브젝트 : 블럭 오브젝트는 문자열 리스트이다.
 * - 라인 오브젝트 : 라인 오브젝트는 블럭 오브젝트의 리스트이다.
 * - 결과 오브젝트 : 결과 오브젝트는 라인 오브젝트의 리스트이다.
 * - 각 오브젝트의 관계는 다음의 예시를 통해 확인할 수 있다.
 * -# @e "hello" 한 단어를 필기 했다면, 다음과 같이 결과 오브젝트에 인식 결과가 저장된다.\n
 * 사용자의 필기에 따라 후보 문자와 개수는 달라질 수 있다.
 * 1순위 후보가 best 후보가 된다.
 * @code
 * Result = { Line[0] }
 *     Line[0] = { Block[0] }
 *         Block[0] = { "hello", "hell", ... } // 1순위 후보: hello, 2순위 후보: hell, 3순위 후보: ...
 * @endcode
 * -# @e "hello world" 한 문장을 필기 했다면, 다음과 같이 결과 오브젝트에 인식 결과가 저장된다.
 * @code
 * Result = { Line[0] }
 *     Line[0] = { Block[0], Block[1] }
 *         Block[0] = { "hello", "hell", ... }
 *         Block[1] = { "world", "word", ... }
 * @endcode
 * -# @e "hello world" @e "good morning" 두 문장을 두 줄로 필기 했다면, 다음과 같이 결과 오브젝트에 인식 결과가 저장된다.
 * @code
 * Result = { Line[0], Line[1] }
 *     Line[0] = { Block[0], Block[1] }
 *         Block[0] = { "hello", "hell", ... }
 *         Block[1] = { "world", "word", ... }
 *     Line[1] = { Block[0], Block[1] }
 *         Block[0] = { "good", "good", ... }
 *         Block[1] = { "morning", "morming", ... }
 * @endcode
 */

/*@{*/

//! Object Handle
typedef const void* DHWRResultObject;
//! Line Handle
typedef const void* DHWRResultLine;
//! Block Handle
typedef const void* DHWRResultBlock;

//!  블럭의 정보를 저장
typedef struct tagDHWRBlockInfo {
  //! 블럭 내의 문자열을 구성하는 획의 개수 (ex "hello" --> stroke count = 5)
  int stroke_count;
  //! 획의 인덱스 포인터 @ref DHWRGetStrokeIndices
  int* stroke_indices;
} DHWRBlockInfo;

/*!
 *  @brief 결과 오브젝트를 생성한다
 *  @return @ref DHWRResultObject
 */
DHWR_API DHWRResultObject DHWRCreateResultObject(void);

/*!
 *  @brief 결과 오브젝트를 제거한다
 *  @param [in] result 결과 오브젝트
 *  @return void
 */
DHWR_API void DHWRDestroyResultObject(DHWRResultObject result);

/*!
 *  @brief 결과 값의 라인 크기를 가져온다
 *  @param [in] result 결과 오브젝트
 *  @return Line Size
 */
DHWR_API unsigned int DHWRGetLineSize(DHWRResultObject result);

/*!
 *  @brief 지정된 인덱스의 라인 결과를 가져온다
 *  @param [in] result 결과 오브젝트
 *  @param [in] index 라인 인덱스
 *  @return @ref DHWRResultLine
 */
DHWR_API DHWRResultLine DHWRGetLine(DHWRResultObject result, const unsigned int index);

/*!
 *  @brief 라인 결과 값의 블럭 크기를 가져온다
 *  @param [in] line 라인 결과 값
 *  @return Block Size
 */
DHWR_API unsigned int DHWRGetBlockSize(DHWRResultLine line);

/*!
 *  @brief 라인 결과값의 지정된 인덱스의 블럭 결과를 가져온다
 *  @param [in] line 라인
 *  @param [in] index 블럭 인덱스
 *  @return @ref DHWRResultBlock
 */
DHWR_API DHWRResultBlock DHWRGetBlock(DHWRResultLine line, const unsigned int index);

/*!
 *  @brief 블럭 결과값의 처리된 획의 개수를 얻어온다
 *  @param [in] block 블럭 결과 값
 *  @return Stroke Size
 */
DHWR_API unsigned int DHWRGetStrokeSize(DHWRResultBlock block);

/*!
 *  @brief 블럭 결과값의 처리된 획의 인덱스들을 얻어온다
 *  @param [in] block 블럭 결과 값
 *  @param [out] indices 인덱스 포인터
 *  @param [in] size 인덱스 포인터 사이즈
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned int DHWRGetStrokeIndices(DHWRResultBlock block, int* indices, int size);

/*!
 *  @brief 블럭 결과값의 인식점수를 얻어온다
 *  @param [in] block 블럭 결과 값
 *  @param [out] scores 인식점수 포인터
 *  @param [in] size 인식점수 포인터 사이즈
 *  @return @ref DHWR_ERROR_CODE "Error Code"
 */
DHWR_API unsigned int DHWRGetCandidateScores(DHWRResultBlock block, float* scores, int size);

/*!
 *  @brief 블럭 결과 값에서 후보 크기를 가져온다.
 *  @param [in] block 블럭 결과 값
 *  @return Candidate Size
 */
DHWR_API unsigned int DHWRGetCandidateSize(DHWRResultBlock block);

/*!
 *  @brief 블럭 결과 값에서 인식 후보를 가져온다.
 *  @param [in] block 블럭 결과 값
 *  @param [in] index 후보 인덱스
 *  @param [out] length 인식후보 문자열 크기
 *  @return 인식 후보 문자열
 */
DHWR_API const wchar_t* DHWRGetCandidate(DHWRResultBlock block,
    const unsigned int index, unsigned int* length);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif // __DHWR_RESULT_OBJECT_H__
